@@ -16,7 +16,7 @@
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.  '''
 
-__version__ = "0.3.06"
+__version__ = "0.3.07"
 __author__ = "nagev"
 __license__ = "GPLv3"
 
@@ -25,12 +25,27 @@ import sys
 import time
 import json
 import logging
-import urllib
-import urllib2
-from urlparse import parse_qs
+
+def decode_if_py3(data):
+    return data.decode("UTF-8")
+
+if sys.version_info[:2] >= (3, 0):
+    from urllib.request import build_opener
+    from urllib.parse import parse_qs, unquote_plus
+else:
+    decode_if_py3 = lambda x: x
+    from urllib2 import build_opener
+    from urllib import unquote_plus
+    from urlparse import parse_qs
 
 #logging.basicConfig(level=logging.DEBUG)
 
+def _extract_function_from_js(funcname, js):
+    # Find a function definition named funcname and extract components
+    match = re.search(r'function %s\(((?:\w+,?)+)\)\{([^\{]+)\}' % funcname, js)
+    return {'name': funcname, 'argnames': match.group(1).split(","),
+        'body': match.group(2) }
+
 def _getval(val, argsdict): # resolves variable values. preserves int literals
     match = re.match(r'(\d+)', val)
     if match:
@@ -40,6 +55,17 @@ def _getval(val, argsdict): # resolves variable values. preserves int literals
     else:
         raise RuntimeError("Error val %s from dict %s" % (val, argsdict))
 
+def _get_func_from_call(f_old, fname, argscall, js):
+    argscall = argscall.split(",")
+    newfunc = _extract_function_from_js(fname, js)
+    newfunc['args'] = {}
+    for n, argname in enumerate(argscall):
+        value = _getval(argname, f_old['args'])
+        # curvar is the argument name specified in the new function definition
+        curvar = newfunc['argnames'][n]
+        newfunc['args'][curvar] = value
+    return newfunc
+
 def _solve(f, js):
     # solve basic javascript function
     parts = f['body'].split(";")
@@ -52,14 +78,7 @@ def _solve(f, js):
         m = re.match(r'(\w+)=(\w+)\(((?:\w+,?)+)\)', part)
         if m: # a function call
             lhs, fname, argscall = m.group(*range(1,4))
-            argscall = argscall.split(",")
-            newfunc = _extract_function_from_js(fname, js)
-            newfunc['args'] = {}
-            for n, argname in enumerate(argscall):
-                value = _getval(argname, f['args'])
-                # curvar is the argname specified in the new func def
-                curvar = newfunc['argnames'][n]
-                newfunc['args'][curvar] = value
+            newfunc = _get_func_from_call(f, fname, argscall, js)
             f['args'][lhs] = _solve(newfunc, js) # recursive call
             continue
         m = re.match(r'var\s(\w+)=(\w+)\[(\w+)\]', part)
@@ -92,24 +111,18 @@ def _solve(f, js):
             continue
         raise RuntimeError("no match for %s" % part)
 
-def _extract_function_from_js(funcname, js):
-    # Find a function named funcname and extract components
-    match = re.search(r'function %s\(((?:\w+,?)+)\)\{([^\{]+)\}' % funcname, js)
-    return {'name': funcname, 'argnames': match.group(1).split(","),
-        'body': match.group(2) }
-
 def _decodesig(sig, js):
     # get main function name from a function call
     sigargument = "g.s"
     sigprefix = "g.sig"
     match = re.search(r'%s\|\|(\w+)\(%s\)' % (sigprefix, sigargument), js)
-    mainfuncname = match.group(1)
-    mainfunction = _extract_function_from_js(mainfuncname, js)
-    if not len(mainfunction['argnames']) == 1:
+    funcname = match.group(1)
+    function = _extract_function_from_js(funcname, js)
+    if not len(function['argnames']) == 1:
         raise RuntimeError("Main sig js function has more than one arg: %s" %
-            mainfunction['argnames'])
-    mainfunction['args'] = {mainfunction['argnames'][0]: sig}
-    return _solve(mainfunction, js)
+            function['argnames'])
+    function['args'] = {function['argnames'][0]: sig}
+    return _solve(function, js)
 
 class Stream():
     resolutions = {
@@ -130,7 +143,7 @@ class Stream():
         '84': ('1280x720-3D', 'mp4'),
         '100': ('640x360-3D', 'webm'),
         '102': ('1280x720-3D', 'webm')}
-
+    
     def __init__(self, streammap, opener, title="ytvid", js=None):
         if not streammap.get("sig", ""):
             logging.debug("Decrypting sig: %s" % streammap['s'])
@@ -151,9 +164,9 @@ class Stream():
 
     def download(self, progress=True, filepath=""):
         response = self._opener.open(self.url)
-        total = int(response.info().getheader('Content-Length').strip())
-        print (u"-Downloading '{}' [{:,} Bytes]".format(self.filename,
-            total)).encode('UTF-8')
+        total = int(response.info()['Content-Length'].strip())
+        print("-Downloading '{}' [{:,} Bytes]".format(self.filename,
+            total))
         status_string = ('  {:,} Bytes [{:.2%}] received. Rate: [{:4.0f} '
                          'kbps].  ETA: [{:.0f} secs]')
         chunksize, bytesdone, t0 = 1024, 0, time.time()
@@ -172,8 +185,8 @@ class Stream():
                 display = (bytesdone, bytesdone * 1.0 / total, rate, eta)
                 status = status_string.format(*display)
                 sys.stdout.write("\r" + status + ' ' * 4 + "\r")
-                sys.stdout.flush
-        print "\nDone"
+                sys.stdout.flush()
+        print("\nDone")
 
 
 class Pafy():
@@ -185,7 +198,7 @@ class Pafy():
         out = ""
         keys = "Title Author ID Duration Rating Views Thumbnail Keywords"
         keys = keys.split(" ")
-        keywords = ", ".join(self.keywords).decode("utf8")
+        keywords = ", ".join(self.keywords)
         length = time.strftime('%H:%M:%S', time.gmtime(self.length))
         info = dict(Title=self.title,
                     Author=self.author,
@@ -200,21 +213,21 @@ class Pafy():
                 out += "%s: %s\n" % (k, info[k])
             except KeyError:
                 pass
-        return out.encode("utf8", "ignore")
+        return out
 
     def _setmetadata(self, allinfo):
-        self.title = allinfo['title'][0].decode('utf-8')
+        self.title = allinfo['title'][0]
         self.author = allinfo['author'][0]
         self.videoid = allinfo['video_id'][0]
-        if 'keywords' in allinfo:
-            self.keywords = allinfo['keywords'][0].split(',')
         self.rating = float(allinfo['avg_rating'][0])
         self.length = int(allinfo['length_seconds'][0])
         self.duration = time.strftime('%H:%M:%S', time.gmtime(self.length))
         self.viewcount = int(allinfo['view_count'][0])
-        self.thumb = urllib.unquote_plus(allinfo['thumbnail_url'][0])
+        self.thumb = unquote_plus(allinfo['thumbnail_url'][0])
         self.formats = allinfo['fmt_list'][0].split(",")
         self.formats = [x.split("/") for x in self.formats]
+        if 'keywords' in allinfo:
+            self.keywords = allinfo['keywords'][0].split(',')
         if allinfo.get('iurlsd'):
             self.bigthumb = allinfo['iurlsd'][0]
         if allinfo.get('iurlmaxres'):
@@ -228,19 +241,19 @@ class Pafy():
         except:
             raise RuntimeError("bad video url")
         infoUrl += vidid + "&asv=3&el=detailpage&hl=en_US"
-        opener = urllib2.build_opener()
+        opener = build_opener()
         ua = ("Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64;"
               "Trident/5.0)")
         opener.addheaders = [('User-Agent', ua)]
         self.keywords = ""
-        allinfo = parse_qs(opener.open(infoUrl).read())
+        allinfo = parse_qs(decode_if_py3(opener.open(infoUrl).read()))
         self._setmetadata(allinfo)
         streamMap = allinfo['url_encoded_fmt_stream_map'][0].split(',')
         smap = [parse_qs(sm) for sm in streamMap]
         js = None
         if not smap[0].get("sig", ""):  # vevo!
             watchurl = "https://www.youtube.com/watch?v=" + vidid
-            watchinfo = opener.open(watchurl).read()
+            watchinfo = opener.open(watchurl).read().decode("UTF-8")
             match = re.search(r';ytplayer.config = ({.*?});', watchinfo)
             try:
                 myjson = json.loads(match.group(1))
@@ -249,7 +262,7 @@ class Pafy():
             args = myjson['args']
             streamMap = args['url_encoded_fmt_stream_map'].split(",")
             html5player = myjson['assets']['js']
-            js = opener.open(html5player).read()
+            js = opener.open(html5player).read().decode("UTF-8")
             smap = [parse_qs(sm) for sm in streamMap]
         self.streams = [Stream(sm, opener, self.title, js) for sm in smap]
 
